View Source ProcessHub.Utility.Extractor (ProcessHub v0.5.0-beta)

Utility module for extracting information from datasets.

Summary

Functions

Extracts child_ids with their child_spec, node_pids, and metadata for local nodes or empty node_pids.

Extracts child_ids with their PIDs of the local nodes.

Functions

Link to this function

local_and_empty_children(hub_id)

@spec local_and_empty_children(ProcessHub.hub_id()) :: %{
  required(ProcessHub.child_id()) =>
    {ProcessHub.child_spec(), [{node(), pid()}],
     ProcessHub.Service.ProcessRegistry.metadata()}
}

Extracts child_ids with their child_spec, node_pids, and metadata for local nodes or empty node_pids.

Link to this function

local_cid_pid_pairs(registry_items)

Extracts child_ids with their PIDs of the local nodes.