View Source ProcessHub.Constant.Event (ProcessHub v0.5.0-beta)

Custom events defined as macros.

Summary

Types

Event used when a node joins the ProcessHub cluster.

Event used when broadcasting local registry data to nodes that join the cluster.

Event used when a node leaves the ProcessHub cluster.

Event used when handling node requests (batched).

Types

@type event_cluster_join() :: :cluster_join_event

Event used when a node joins the ProcessHub cluster.

Link to this type

event_node_registry_broadcast()

@type event_node_registry_broadcast() :: :node_registry_broadcast_event

Event used when broadcasting local registry data to nodes that join the cluster.

Link to this type

event_post_cluster_leave()

@type event_post_cluster_leave() :: :cluster_leave_event

Event used when a node leaves the ProcessHub cluster.

Link to this type

event_requests_handle()

@type event_requests_handle() :: :requests_handle_event

Event used when handling node requests (batched).