View Source ProcessHub.Request.PostAction (ProcessHub v0.5.0-beta)
Represents a post-action to be executed after children are started on the target node.
Post-actions are attached to StartChildrenRequest and executed after children
are successfully started. They use an MFA (module, function, args) pattern to
allow any module to define post-action behavior.
The callback function will be invoked as: apply(m, f, [hub, results | a])
where hub is the Hub struct and results is the list of PostStartData structs.
Summary
Functions
Creates a new PostAction with the given module, function, and additional arguments.