View Source ProcessHub.Service.State (ProcessHub v0.5.0-beta)
The state service provides API functions for managing the state of the hub.
Summary
Functions
Returns a boolean indicating whether the hub cluster is partitioned.
Terminates the local distributed supervisor.
Restarts the local distributed supervisor.
Functions
@spec is_partitioned?(ProcessHub.Hub.t()) :: boolean()
Returns a boolean indicating whether the hub cluster is partitioned.
@spec toggle_quorum_failure(ProcessHub.Hub.t()) :: :ok | {:error, :already_partitioned}
Terminates the local distributed supervisor.
@spec toggle_quorum_success(ProcessHub.Hub.t()) :: :ok | {:error, :not_partitioned}
Restarts the local distributed supervisor.